@@ -502,182 +502,251 @@ def force_line_buffering():
     return stdout0, stderr0
 
 
-def exception_info( exception=None, limit=None, out=None, prefix='', oneline=False):
+def exception_info(
+        exception_or_traceback=None,
+        limit=None,
+        file=None,
+        chain=True,
+        outer=True,
+        _filelinefn=True,
+        ):
     '''
-    General replacement for traceback.* functions that print/return information
-    about exceptions and backtraces. This function provides a simple way of
-    getting the functionality provided by these traceback functions:
+    Shows an exception and/or backtrace.
+
+    Alternative to traceback.* functions that print/return
+    information about exceptions and backtraces, such as:
 
         traceback.format_exc()
         traceback.format_exception()
         traceback.print_exc()
         traceback.print_exception()
 
+    Install as system default with:
+        sys.excepthook = lambda type_, exception, traceback: exception_info( exception, chain='reverse')
+
+    Returns None, or the generated text if <file> is 'return'.
+
     Args:
-        exception:
-            None, or a (type, value, traceback) tuple, e.g. from
-            sys.exc_info(). If None, we call sys.exc_info() and use its return
-            value. If there is no live exception we show information about the
-            current backtrace.
+        exception_or_traceback:
+            None, an Exception or a types.TracebackType. If None we use current
+            exception from sys.exc_info(), otherwise the current backtrace from
+            inspect.stack().
         limit:
-            None or maximum number of stackframes to output.
-        out:
-            None or callable taking single <text> parameter or object with a
-            'write' member that takes a single <text> parameter.
-        prefix:
-            Used to prefix all lines of text.
-        oneline:
-            If true, we only show one line of information.
+            As in traceback.* functions: None to show all frames, positive to
+            show last <limit> frames, negative to exclude outermost -limit
+            frames.
+        file:
+            As in traceback.* functions: file-like object to which we write
+            output, or sys.stderr if None. Special value 'return' makes us
+            return our output as a string.
+        chain:
+            As in traceback.* functions: if true we show chained exceptions as
+            described in PEP-3134. Special value 'because' reverses the usual
+            ordering, showing higher-level exceptions first and joining with
+            'Because:' text.
+        outer:
+            If true (the default) we also show an exception's outer frames
+            above the catch block (see below for details). We use outer=false
+            for chained exceptions to avoid duplication.
+        _filelinefn:
+            Internal only; used with doctest - makes us omit file:line:
+            information to allow simple comparison with expected output.
 
-    Returns:
-        A string containing description of specified exception (if any) and
-        backtrace. Also sends this description to <out> if specified.
+    Differences from traceback.* functions:
 
-    Inclusion of outer frames:
-        We improve upon traceback.* in that we also include outermost stack
-        frames above the point at which an exception was caught - frames from
-        the top-level <module> or thread creation fn to the try..catch block,
-        which makes backtraces much more useful.
+        Frames are displayed as one line in the form:
+            <file>:<line>:<function>: <text>
 
-        Google 'sys.exc_info backtrace incomplete' for more details.
+        Filenames are displayed as relative to the current directory if
+        applicable.
 
-        We separate the two parts of the backtrace using a line '^except
-        raise:'; the idea here is that '^except' is pointing upwards to the
-        frame that caught the exception, while 'raise:' is referring downwards
-        to the frames that eventually raised the exception.
+        Inclusion of outer frames:
+            Unlike traceback.* functions, stack traces for exceptions include
+            outer stack frames above the point at which an exception was caught
+            - frames from the top-level <module> or thread creation to the
+            catch block. [Search for 'sys.exc_info backtrace incomplete' for
+            more details.]
 
-        So the backtrace looks like this:
+            We separate the two parts of the backtrace using a marker line
+            '^except raise:' where '^except' points upwards to the frame that
+            caught the exception and 'raise:' refers downwards to the frame
+            that raised the exception.
 
-            root (e.g. <module> or /usr/lib/python2.7/threading.py:778:__bootstrap():
+            So the backtrace for an exception looks like this:
+
+                <file>:<line>:<fn>: <text>  [in root module.]
+                ...                         [... other frames]
+                <file>:<line>:<fn>: <text>  [the except: block where exception was caught.]
+                ^except raise:              [marker line]
+                <file>:<line>:<fn>: <text>  [try: block.]
+                ...                         [... other frames]
+                <file>:<line>:<fn>: <text>  [where the exception was raised.]
+
+    Examples:
+
+        Define some nested function calls which raise and except and call
+        exception_info(). We use file=sys.stdout so we can check the output
+        with doctest, and set _filelinefn=0 so that the output can be matched
+        easily.
+
+        >>> def a():
+        ...     b()
+        >>> def b():
+        ...     try:
+        ...         c()
+        ...     except Exception as e:
+        ...         exception_info( file=sys.stdout, chain=g_chain, _filelinefn=0)
+        >>> def c():
+        ...     try:
+        ...         d()
+        ...     except Exception as e:
+        ...         raise Exception( 'c: d() failed') from e
+        >>> def d():
+        ...     e()
+        >>> def e():
+        ...     raise Exception('e(): deliberate error')
+
+        We use +ELLIPSIS to allow '...' to match arbitrary outer frames from
+        the doctest code itself.
+
+        With chain=True (the default), we output low-level exceptions first,
+        matching the behaviour of traceback.* functions:
+
+        >>> g_chain = True
+        >>> a() # doctest: +REPORT_UDIFF +ELLIPSIS
+        Traceback (most recent call last):
+            c(): d()
+            d(): e()
+            e(): raise Exception('e(): deliberate error')
+        Exception: e(): deliberate error
+        <BLANKLINE>
+        The above exception was the direct cause of the following exception:
+        Traceback (most recent call last):
             ...
-            file:line in the except: block where the exception was caught.
+            <module>(): a() # doctest: +REPORT_UDIFF +ELLIPSIS
+            a(): b()
+            b(): exception_info( file=sys.stdout, chain=g_chain, _filelinefn=0)
             ^except raise:
-            file:line in the try: block.
+            b(): c()
+            c(): raise Exception( 'c: d() failed') from e
+        Exception: c: d() failed
+
+
+        With chain='because', we output high-level exceptions first:
+
+        >>> g_chain = 'because'
+        >>> a() # doctest: +REPORT_UDIFF +ELLIPSIS
+        Traceback (most recent call last):
             ...
-            file:line where the exception was raised.
-
-        The items below the '^except raise:' marker are the usual items that
-        traceback.* shows for an exception.
-
-    Also the backtraces that are generated are more concise than those provided
-    by traceback.* - just one line per frame instead of two - and filenames are
-    output relative to the current directory if applicable. And one can easily
-    prefix all lines with a specified string, e.g. to indent the text.
+            <module>(): a() # doctest: +REPORT_UDIFF +ELLIPSIS
+            a(): b()
+            b(): exception_info( file=sys.stdout, chain=g_chain, _filelinefn=0)
+            ^except raise:
+            b(): c()
+            c(): raise Exception( 'c: d() failed') from e
+        Exception: c: d() failed
+        <BLANKLINE>
+        Because:
+        Traceback (most recent call last):
+            c(): d()
+            d(): e()
+            e(): raise Exception('e(): deliberate error')
+        Exception: e(): deliberate error
     '''
-    if exception is None:
-        exception = sys.exc_info()
-    etype, value, tb = exception
-    out2 = io.StringIO()
-    try:
-        frames = []
+    if isinstance( exception_or_traceback, types.TracebackType):
+        exception = None
+        tb = exception_or_traceback
+    elif isinstance( exception_or_traceback, BaseException):
+        exception = exception_or_traceback
+    elif exception_or_traceback:
+        assert 0, f'Unrecognised exception_or_traceback type: {type(exception_or_traceback)}'
+    else:
+        _, exception, tb = sys.exc_info()
+        if not exception:
+            tb = inspect.stack()[1:]
 
-        def append_frame(f):
-            f4 = f[4]
-            f4 = f4[0].strip() if f4 else ''
-            ff = f[1], f[2], f[3], f4
-            frames.append(ff)
+    if file == 'return':
+        out = io.StringIO()
+    else:
+        out = file if file else sys.stderr
 
-        if tb:
-            # There is a live exception.
-            #
-            # Get frames above point at which exception was caught - frames
-            # starting at top-level <module> or thread creation fn, and ending
-            # at the point in the catch: block from which we were called.
-            #
-            # These frames are not included explicitly in sys.exc_info()[2] and are
-            # also omitted by traceback.* functions, which makes for incomplete
-            # backtraces that miss much useful information.
-            #
-            for f in reversed(inspect.getouterframes(tb.tb_frame)):
-                append_frame(f)
-        else:
-            # No exception; use current backtrace.
-            for f in inspect.stack():
-                append_frame(f)
+    def do_chain( exception):
+        exception_info( exception, limit, out, chain, outer=False, _filelinefn=_filelinefn)
 
-        # If there is a live exception, append frames from point in the try:
-        # block that caused the exception to be raised, to the point at which
-        # the exception was thrown.
-        #
-        # [One can get similar information using traceback.extract_tb(tb):
-        #   for f in traceback.extract_tb(tb):
-        #       frames.append(f)
-        # ]
-        if tb:
-            # Insert a marker to separate the two parts of the backtrace, used
-            # for our special '^except raise:' line.
-            frames.append( None)
+    if exception and chain and chain != 'because':
+        if exception.__cause__:
+            do_chain( exception.__cause__)
+            out.write( '\nThe above exception was the direct cause of the following exception:\n')
+        elif exception.__context__:
+            do_chain( exception.__context__)
+            out.write( '\nDuring handling of the above exception, another exception occurred:\n')
 
-            for f in inspect.getinnerframes(tb):
-                append_frame(f)
+    cwd = os.getcwd() + os.sep
 
-        cwd = os.getcwd() + os.sep
-        if oneline:
-            if etype and value:
-                # The 'exception_text' variable below will usually be assigned
-                # something like '<ExceptionType>: <ExceptionValue>', unless
-                # there was no explanatory text provided (e.g. "raise Exception()").
-                # In this case, str(value) will evaluate to ''.
-                exception_text = traceback.format_exception_only(etype, value)[0].strip()
-                filename, line, fnname, text = frames[-1]
-                if filename.startswith(cwd):
-                    filename = filename[len(cwd):]
-                if not str(value):
-                    # The exception doesn't have any useful explanatory text
-                    # (for example, maybe it was raised by an expression like
-                    # "assert <expression>" without a subsequent comma).  In
-                    # the absence of anything more helpful, print the code that
-                    # raised the exception.
-                    exception_text += ' (%s)' % text
-                line = '%s%s at %s:%s:%s()' % (prefix, exception_text, filename, line, fnname)
-                out2.write(line)
-        else:
-            out2.write( '%sBacktrace:\n' % prefix)
-            for frame in frames:
-                if frame is None:
-                    out2.write( '%s    ^except raise:\n' % prefix)
-                    continue
-                filename, line, fnname, text = frame
-                if filename.startswith( cwd):
-                    filename = filename[ len(cwd):]
-                if filename.startswith( './'):
-                    filename = filename[ 2:]
-                out2.write( '%s    %s:%s:%s(): %s\n' % (
-                        prefix, filename, line, fnname, text))
+    def output_frames( frames, reverse, limit):
+        if reverse:
+            frames = reversed( frames)
+        if limit is not None:
+            frames = list( frames)
+            frames = frames[ -limit:]
+        for frame in frames:
+            f, filename, line, fnname, text, index = frame
+            text = text[0].strip() if text else ''
+            if filename.startswith( cwd):
+                filename = filename[ len(cwd):]
+            if filename.startswith( f'.{os.sep}'):
+                filename = filename[ 2:]
+            if _filelinefn:
+                out.write( f'    {filename}:{line}:{fnname}(): {text}\n')
+            else:
+                out.write( f'    {fnname}(): {text}\n')
 
-            if etype and value:
-                out2.write( '%sException:\n' % prefix)
-                lines = traceback.format_exception_only( etype, value)
-                for line in lines:
-                    # It seems that the lines returned from
-                    # traceback.format_exception_only() can sometimes contain
-                    # \n characters, so we do an additional loop to ensure that
-                    # these are indented consistently.
-                    #
-                    for line2 in line.split('\n'):
-                        out2.write( '%s    %s\n' % ( prefix, line2))
+    out.write( 'Traceback (most recent call last):\n')
+    if exception:
+        tb = exception.__traceback__
+        if outer:
+            output_frames( inspect.getouterframes( tb.tb_frame), reverse=True, limit=limit)
+            out.write( '    ^except raise:\n')
+        output_frames( inspect.getinnerframes( tb), reverse=False, limit=None)
+    else:
+        output_frames( tb, reverse=True, limit=limit)
 
-        text = out2.getvalue()
+    if exception:
+        lines = traceback.format_exception_only( type(exception), exception)
+        for line in lines:
+            out.write( line)
 
-        # Write text to <out> if specified.
-        out = getattr( out, 'write', out)
-        if callable( out):
-            out( text)
-        return text
+    if exception and chain == 'because':
+        if exception.__cause__:
+            out.write( '\nBecause:\n')
+            do_chain( exception.__cause__)
+        elif exception.__context__:
+            out.write( '\nBecause error occurred handling this exception:\n')
+            do_chain( exception.__context__)
 
-    finally:
-        # clear things to avoid cycles.
-        del exception
-        del etype
-        del value
-        del tb
-        del frames
+    if file == 'return':
+        return out.getvalue()
 
 
 def number_sep( s):
     '''
     Simple number formatter, adds commas in-between thousands. <s> can
     be a number or a string. Returns a string.
+
+    >>> number_sep(1)
+    '1'
+    >>> number_sep(12)
+    '12'
+    >>> number_sep(123)
+    '123'
+    >>> number_sep(1234)
+    '1,234'
+    >>> number_sep(12345)
+    '12,345'
+    >>> number_sep(123456)
+    '123,456'
+    >>> number_sep(1234567)
+    '1,234,567'
     '''
     if not isinstance( s, str):
         s = str( s)
@@ -696,14 +765,6 @@ def number_sep( s):
     ret += s[end:]
     return ret
 
-assert number_sep(1)=='1'
-assert number_sep(12)=='12'
-assert number_sep(123)=='123'
-assert number_sep(1234)=='1,234'
-assert number_sep(12345)=='12,345'
-assert number_sep(123456)=='123,456'
-assert number_sep(1234567)=='1,234,567'
-
 
 class Stream:
     '''
