@@ -729,16 +729,13 @@ Usage:
             Downloads swig if not present in current directory, extracts
             swig.exe and sets things up to use it subsequently.
 
-        --sync [-d] <destination>
-            Use rsync to copy C++ source generated by action=0 to
-            <destination>. Also copies syntax-coloured .html versions.
+        --sync-docs <destination>
+            Use rsync to copy contents of docs/generated/ to remote destination.
 
-            If '-d' is specified, also sync all files generated by --doc
-            into a remote location. Also uploads a small top-level generated
-            index.html file containing links to the C, C++ and Python pages
-            along with information on the date and git id for the generation.
-
-            E.g. --sync -d julian@casper.ghostscript.com:~julian/public_html/mupdf-bindings/
+        --sync-pretty <destination>
+            Use rsync to copy generated C++ and Python files to <destination>. Also
+            uses generates and copies .html versions of these files that use
+            run_prettify.js from cdn.jsdelivr.net to show embelished content.
 
         --test-csharp
             Tests the experimental C# API.
@@ -1540,6 +1537,155 @@ def csharp_settings(build_dirs):
     return csc, mono, mupdf_cs
 
 
+def make_docs( build_dirs, languages):
+
+    def do_doxygen( name, outdir, path):
+        '''
+        name:
+            Doxygen PROJECT_NAME of generated documentation
+        outdir:
+            Directory in which we run doxygen, so root of generated
+            documentation will be in <outdir>/html/index.html
+        path:
+            Doxygen INPUT setting; this is the path of the directory which
+            contains the API to document. If a relative path, it should be
+            relative to <outdir>.
+        '''
+        # We generate a blank doxygen configuration file, make
+        # some minimal changes, then run doxygen on the modified
+        # configuration.
+        #
+        assert 'docs/generated/' in outdir
+        jlib.ensure_empty_dir( outdir)
+        dname = f'{name}.doxygen'
+        dname2 = os.path.join( outdir, dname)
+        jlib.system( f'cd {outdir}; rm -f {dname}0; doxygen -g {dname}0', out='return')
+        with open( dname2+'0') as f:
+            dtext = f.read()
+        dtext, n = re.subn( '\nPROJECT_NAME *=.*\n', f'\nPROJECT_NAME = {name}\n', dtext)
+        assert n == 1
+        dtext, n = re.subn( '\nEXTRACT_ALL *=.*\n', f'\nEXTRACT_ALL = YES\n', dtext)
+        assert n == 1
+        dtext, n = re.subn( '\nINPUT *=.*\n', f'\nINPUT = {path}\n', dtext)
+        assert n == 1
+        dtext, n = re.subn( '\nRECURSIVE *=.*\n', f'\nRECURSIVE = YES\n', dtext)
+        with open( dname2, 'w') as f:
+            f.write( dtext)
+        #jlib.system( f'diff -u {dname2}0 {dname2}', raise_errors=False)
+        command = f'cd {outdir}; doxygen {dname}'
+        jlib.system( command, out='return', verbose=1)
+        jlib.log( 'have created: {outdir}/html/index.html')
+
+    out_dir = f'{build_dirs.dir_mupdf}/docs/generated'
+
+    for language in languages:
+
+        if language == 'c':
+            do_doxygen( 'mupdf', f'{out_dir}/c', f'{build_dirs.dir_mupdf}/include')
+
+        elif language == 'c++':
+            do_doxygen( 'mupdfcpp', f'{out_dir}/c++', f'{build_dirs.dir_mupdf}/platform/c++/include')
+
+        elif language == 'python':
+            ld_library_path = os.path.abspath( f'{build_dirs.dir_so}')
+            jlib.ensure_empty_dir( f'{out_dir}/python')
+            pythonpath = os.path.relpath( f'{build_dirs.dir_so}', f'{out_dir}/python')
+            input_relpath = os.path.relpath( f'{build_dirs.dir_so}/mupdf.py', f'{out_dir}/python')
+            jlib.system(
+                    f'cd {out_dir}/python && LD_LIBRARY_PATH={ld_library_path} PYTHONPATH={pythonpath} pydoc3 -w {input_relpath}',
+                    out='log',
+                    verbose=True,
+                    )
+            path = f'{out_dir}/python/mupdf.html'
+            assert os.path.isfile( path)
+
+            # Add some styling.
+            #
+            with open( path) as f:
+                text = f.read()
+
+            m1 = re.search( '[<]/head[>][<]body[^>]*[>]\n', text)
+            m2 = re.search( '[<]/body[>]', text)
+            assert m1
+            assert m2
+            #jlib.log( '{=m1.start() m1.end() m2.start() m2.end()}')
+
+            a = text[ : m1.start()]
+            b = textwrap.dedent('''
+                    <link href="../../../../../css/default.css" rel="stylesheet" type="text/css" />
+                    <link href="../../../../../css/language-bindings.css" rel="stylesheet" type="text/css" />
+                    ''')
+            c = text[ m1.start() : m1.end()]
+            d = textwrap.dedent('''
+                    <main style="display:block;">
+                        <a class="no-underline" href="../../../index.html">
+                            <div class="banner" role="heading" aria-level="1">
+                                <h1>MuPDF Python bindings</h1>
+                            </div>
+                        </a>
+                        <div class="outer">
+                            <div class="inner">
+                    ''')
+            e = text[ m1.end() : m2.end()]
+            f = textwrap.dedent('''
+                    </div></div>
+                    </main>
+                    ''')
+            g = text[ m2.end() : ]
+            text = a + b + c + d + e + f + g
+            with open( path, 'w') as f:
+                f.write( text)
+            jlib.log( 'have created: {path}')
+
+        else:
+            raise Exception( f'unrecognised language param: {lang}')
+
+    # Create index.html with links to the different bindings'
+    # documentation.
+    #
+    #mupdf_dir = os.path.abspath( f'{__file__}/../../..')
+    top_index_html = f'{out_dir}/index.html'
+    with open( top_index_html, 'w') as f:
+        git_id = jlib.get_git_id( build_dirs.dir_mupdf)
+        git_id = git_id.split( '\n')[0]
+        f.write( textwrap.dedent( f'''
+                <!DOCTYPE html>
+
+                <html lang="en">
+                    <head>
+                        <link href="../../css/default.css" rel="stylesheet" type="text/css" />
+                        <link href="../../css/language-bindings.css" rel="stylesheet" type="text/css" />
+                    </head>
+                    <body>
+                        <main style="display:block;">
+                            <div class="banner" role="heading" aria-level="1">
+                                <h1>MuPDF bindings</h1>
+                            </div>
+                            <div class="outer">
+                                <div class="inner">
+                                    <ul>
+                                        <li><a href="c/html/index.html">C</a> (generated by Doxygen).
+                                        <li><a href="c++/html/index.html">C++</a> (generated by Doxygen).
+                                        <li><a href="python/mupdf.html">Python</a> (generated by Pydoc).
+                                    </ul>
+                                    <small>
+                                        <p>Generation:</p>
+                                        <ul>
+                                            <li>Date: {jlib.date_time()}
+                                            <li>Git: {git_id}
+                                        </ul>
+                                    </small>
+                                </div>
+                            </div>
+                        </main>
+                    </body>
+                </html>
+                '''
+                ))
+    jlib.log( 'Have created: {top_index_html}')
+
+
+
 def main2():
 
     # Set default build directory. Can br overridden by '-d'.
@@ -1604,69 +1750,11 @@ def main2():
                             jlib.system( command, out='log', raise_errors=False)
 
             elif arg == '--doc':
-
-                def do_doxygen( name, outdir, path):
-                    '''
-                    name:
-                        Doxygen PROJECT_NAME of generated documentation
-                    outdir:
-                        Directory in which we run doxygen, so root of generated
-                        documentation will be in <outdir>/html/index.html
-                    path:
-                        Doxygen INPUT setting; this is the path relative to
-                        <outdir> of the directory which contains the API to
-                        document.
-                    '''
-                    # We generate a blank doxygen configuration file, make
-                    # some minimal changes, then run doxygen on the modified
-                    # configuration.
-                    #
-                    dname = f'{name}.doxygen'
-                    dname2 = os.path.join( outdir, dname)
-                    jlib.system( f'cd {outdir}; rm -f {dname}0; doxygen -g {dname}0', out='return')
-                    with open( dname2+'0') as f:
-                        dtext = f.read()
-                    dtext, n = re.subn( '\nPROJECT_NAME *=.*\n', f'\nPROJECT_NAME = {name}\n', dtext)
-                    assert n == 1
-                    dtext, n = re.subn( '\nEXTRACT_ALL *=.*\n', f'\nEXTRACT_ALL = YES\n', dtext)
-                    assert n == 1
-                    dtext, n = re.subn( '\nINPUT *=.*\n', f'\nINPUT = {path}\n', dtext)
-                    assert n == 1
-                    dtext, n = re.subn( '\nRECURSIVE *=.*\n', f'\nRECURSIVE = YES\n', dtext)
-                    with open( dname2, 'w') as f:
-                        f.write( dtext)
-                    #jlib.system( f'diff -u {dname2}0 {dname2}', raise_errors=False)
-                    command = f'cd {outdir}; doxygen {dname}'
-                    jlib.log( 'running: {command}')
-                    jlib.system( command, out='return')
-                    jlib.log( 'have created: {outdir}/html/index.html')
-
-                langs = args.next()
-                if langs == 'all':
-                    langs = 'c,c++,python'
-
-                langs = langs.split( ',')
-
-                for lang in langs:
-                    if lang == 'c':
-                        do_doxygen( 'mupdf', 'include', 'mupdf')
-
-                    elif lang == 'c++':
-                        do_doxygen( 'mupdfcpp', 'platform/c++/include', 'mupdf')
-
-                    elif lang == 'python':
-                        ld_library_path = os.path.abspath( f'{build_dirs.dir_so}')
-                        pythonpath = build_dirs.dir_so
-                        jlib.system(
-                                f'cd {build_dirs.dir_so}; LD_LIBRARY_PATH={ld_library_path} PYTHONPATH={pythonpath} pydoc3 -w ./mupdf.py',
-                                out='log',
-                                )
-                        path = f'{build_dirs.dir_so}/mupdf.html'
-                        assert os.path.isfile( path)
-                        jlib.log( 'have created: {path}')
-
-                    else:
-                        raise Exception( f'unrecognised language param: {lang}')
+                languages = args.next()
+                if languages == 'all':
+                    languages = 'c,c++,python'
+                languages = languages.split( ',')
+                make_docs( build_dirs, languages)
 
             elif arg == '--ref':
                 assert 'mupdfwrap_ref' in build_dirs.ref_dir
@@ -1808,12 +1896,9 @@ def main2():
                 else:
                     jlib.log('Ignoring {arg} because not running on Windows')
 
-            elif arg == '--sync':
+            elif arg == '--sync-pretty':
                 sync_docs = False
                 destination = args.next()
-                if destination == '-d':
-                    sync_docs = True
-                    destination = args.next()
                 jlib.log( 'Syncing to {destination=}')
                 generated = cpp.Generated(f'{build_dirs.dir_mupdf}/platform/c++')
                 files = generated.h_files + generated.cpp_files + [
@@ -1842,41 +1927,6 @@ def main2():
 
                 files += files_html
 
-                if sync_docs:
-                    top_index_html = '/tmp/index.html'
-                    with open( top_index_html, 'w') as f:
-                        mupdf_dir = os.path.abspath( f'{__file__}/../../..')
-                        git_id = jlib.get_git_id( mupdf_dir)
-                        git_id = git_id.split( '\n')[0]
-                        f.write( textwrap.dedent( f'''
-                                <html>
-                                <body>
-                                <h1>MuPDF bindings</h1>
-                                <ul>
-                                    <li><a href="mupdf/include/html/index.html">C</a> (generated by Doxygen).
-                                    <li><a href="mupdf/platform/c++/include/html/index.html">C++</a> (generated by Doxygen).
-                                    <li><a href="mupdf/build/shared-release/mupdf.html">Python</a> (generated by Pydoc).
-                                </ul>
-                                <small>
-                                <small>
-                                <p>Generation:
-                                <ul>
-                                    <li>Date: {jlib.date_time()}
-                                    <li> Git: {git_id}
-                                </ul>
-                                </small>
-                                </small>
-                                </body>
-                                </html>
-                                '''
-                                ))
-                    files += [
-                            f'{build_dirs.dir_mupdf}/include/html/',
-                            f'{build_dirs.dir_mupdf}/platform/c++/include/html/',
-                            f'{build_dirs.dir_so}/mupdf.html',
-                            f'{top_index_html}',
-                            ]
-
                 # Insert extra './' into each path so that rsync -R uses the
                 # 'mupdf/...' tail of each local path for the remote path.
                 #
@@ -1886,6 +1936,16 @@ def main2():
 
                 jlib.system( f'rsync -aiRz {" ".join( files)} {destination}', verbose=1, out='log')
 
+            elif arg == '--sync-docs':
+                # We use extra './' so that -R uses remaining path on
+                # destination.
+                #
+                destination = args.next()
+                jlib.system( f'rsync -aiRz {build_dirs.dir_mupdf}/docs/generated/./ {destination}', verbose=1, out='log')
+
+            elif arg == '--test-cpp':
+                cpp.test()
+
             elif arg in ('--test-python', '-t', '--test-python-gui'):
 
                 env_extra, command_prefix = python_settings(build_dirs)
@@ -2169,7 +2229,7 @@ def main():
     try:
         main2()
     except Exception:
-        sys.stderr.write(jlib.exception_info())
+        jlib.exception_info()
         sys.exit(1)
 
 if __name__ == '__main__':
