@@ -68,7 +68,13 @@ def build_swig(
     assert language in ('python', 'csharp')
     # Find version of swig. (We use quotes around <swig> to make things work on
     # Windows.)
-    t = jlib.system( f'"{swig_command}" -version', out='return')
+    try:
+        t = jlib.system( f'"{swig_command}" -version', out='return')
+    except Exception as e:
+        if state_.windows:
+            raise Exception( 'swig failed; on Windows swig can be auto-installed with: --swig-windows-auto') from e
+        else:
+            raise
     m = re.search( 'SWIG Version ([0-9]+)[.]([0-9]+)[.]([0-9]+)', t)
     assert m
     swig_major = int( m.group(1))
@@ -875,6 +881,10 @@ def build_swig(
                 # work on Linux.
                 prefix = textwrap.dedent(
                         f'''
+                        import ctypes
+                        import os
+                        import importlib
+
                         # The required .so's are in the same directory as this
                         # Python file. On OpenBSD we can explicitly load these
                         # .so's here using ctypes.cdll.LoadLibrary(), which
@@ -882,9 +892,6 @@ def build_swig(
                         #
                         # Unfortunately this doesn't work on Linux.
                         #
-                        import ctypes
-                        import os
-                        import importlib
                         for leaf in ('libmupdf.so', 'libmupdfcpp.so', '_mupdf.so'):
                             path = os.path.abspath(f'{{__file__}}/../{{leaf}}')
                             #print(f'path={{path}}')
